(**
 * Client handling functions
 * @author Jonathan Bryant
 *)

type 'a t
(** The abstract type of clients *)

val create : Unix.file_descr -> 'a t
(** [Client.create fd] creates a client using the UNIX file descriptor [fd].
 *  Note that this file descriptor should have already had [Unix.connect]
 *  applied to it (this is for extensibility reasons). *)

val deserializer : (in_channel -> 'a) option -> 'a t -> 'a t
(** [Client.deserializer f c] sets the deserialization function of the client
 *  [c] to the function [f].  If this function is applied to a [None] value,
 *  then the client reverts to the default deserialization function.  Raises
 *  {!ClientServer.Invalid_state} if the client is already connected. *)

val handler : ('a -> 'a option) option -> 'a t -> 'a t
(** [Client.handler f c] sets the handler function of the client [c] to the
 *  function [f].
 *
 *  If this function is applied to a [None] value, then user of
 *  this module must manually communicate with the server via the {!Client.send}
 *  and {!Client.receive} functions.  If this is applied to a [Some f] value,
 *  then the function [f] is applied all received messages.  If [f] returns a
 *  [Some m] value, then the message [m] is sent back to the server.
 *
 *  Raises {!ClientServer.Invalid_state} if the client is already connected. *)

val serializer : (out_channel -> 'a -> unit) option -> 'a t -> 'a t
(** [Client.serializer f c] sets the serialization function of the client [c] to
 *  the function [f].  If this function is applied to a [None] value, then the
 *  client reverts to the default serialization function.  Raises
 *  {!ClientServer.Invalid_state} if the client is already connected. *)

val connect : 'a t -> 'a t
(** [Client.connect c] connects the client [c] using the UNIX file descriptor
 *  specified when the client was created.  If a handler function has been
 *  assigned using the {!Client.handler} function, then this client is run on a
 *  separate thread.  Otherwise, it is run within the current thread.  Raises
 *  {!ClientServer.Invalid_state} if the client is already started. *)

val send : 'a t -> 'a -> unit
(** [Client.send c m] sends the message [m] to the server that the client [c] is
 *  connected to.  Raises {!ClientServer.Invalid_state} if the client is not
 *  connected or if a handler function is set. *)

val receive : 'a t -> 'a
(** [Client.receive c] receives a message from the server that the client [c] is
 *  connected to.  Raises {!ClientServer.Invalid_state} if the client is not
 *  connected of if a handler function is set. *)

val poll : 'a t -> 'a option
(** [Client.poll c] receives a message from the server that the client [c] is
 *  connected to.  If there is not a message pending, this function returns
 *  [None].  Raises {!ClientServer.Invalid_state} if the client is not
 *  connected or if a handler function is set. *)

val disconnect : 'a t -> 'a t
(** [Client.disconnect c] disconnects the client [c] from the server.  Raises
 *  {!ClientServer.Invalid_state} if the client is not connected. *)

