(**
 * This is a generic multi-threaded, extensible client/server framework for
 * OCaml.
 *
 * It is comprised of six modules:
 * - {!ClientServer} which contains common data and most of the documentation
 * - {!EventBroker} which manages the events passed back and forth between the
 * threads.  This module is independent of the rest of the modules, so it can be
 * reused elsewhere.  It is sort of like an asynchrounous version of the [Event]
 * module included in the standard library.
 * - {!Connection} which has functions for manipulating connections to the
 * server.
 * - {!Acceptor} which has functions for the basic control of the acceptor
 * thread.
 * - {!Server} which is the main module for controlling the server thread.
 * - {!Client} which is the main module for controlling a client.
 *
 * Although the exact hooks are explained in their specific modules, in general
 * there are three types of hooks:
 * - <b>(De)Serialization </b> hooks for defining how data is to be read off of
 * and writen onto the input and output channels.  By default, these simply use
 * the [Marshal] module to send the data, but these could be replaced by, for
 * example, HTTP header parsers and generators to create a HTTP server.
 * - <b>Message Handling</b> hooks for defining the default behavior of the
 * client or server.  By default, these hooks simply drop all incoming messages.
 * - <b>Connection Lifecycle</b> hooks for adding behavior when connections are
 * created or destroyed.  By default, these functions do nothing.
 *
 * Todo List:
 * - Add thread pooling for better response time and lower resource usage
 * - Add the option to use heavyweight processes instead of threads
 * - Add the option to use both heavy- and lightweight processes with multiple
 * thread pools (like Apache does), hopefully with a prefork ability (also like
 * Apache)
 * 
 * @author Jonathan Bryant
 *)

exception Invalid_state of string
(** An exception to be raised when either the client or the server is about to
 *  be put into an invalid state. *)

type 'a message =
| Connect of Unix.file_descr (** A connection has been accepted *)
| Disconnect of int (** A connection has been lost *)
| Quit (** A thread should terminate *)
| Join of Thread.t (** The receiving thread should join this thread *)
| Send of 'a (** The receiving {!Connection} should send this message *)
(** Internal messages passed among threads *)
