(**
 * Functions for manipulating individual connections to the server.
 * @author Jonathan Bryant
 *)

type 'a t
(** The abstract type of connections to a server *)

val create : 'a ClientServer.message EventBroker.t -> int -> Unix.file_descr -> 'a t
(** [Connection.create eb q fd] creates a connection using the {!EventBroker}
 *  [eb], the queue [q], and the UNIX file descriptor [fd].  *)

val id : 'a t -> int
(** [Connection.id c] gets the identification number of the connection [c]. *)

val deserializer : (in_channel -> 'a) option -> 'a t -> 'a t
(** [Connection.deserializer f c] sets the deserialization function of the
 *  connection [c] to the function [f].  If this function is applied to a [None]
 *  value then the connection reverts to its default behavior.  Raises
 *  {!ClientServer.Invalid_state} if the connection is already started. *)

val handler : ('a -> 'a option) option -> 'a t -> 'a t
(** [Connection.handler f c] sets the message handling function of the
 *  connection [c] to the function [f].  If this function is applied to a [None]
 *  value then the connection reverts to its default behavior.  Raises
 *  {!ClientServer.Invalid_state} if the connection is already started. *)

val serializer : (out_channel -> 'a -> unit) option -> 'a t -> 'a t
(** [Connection.serializer f c] sets the serialization function of the
 *  connection [c] to the function [f].  If this function is applied to a [None]
 *  value then the connection reverts to its default behavior.  Raises
 *  {!ClientServer.Invalid_state} if the connection is already started. *)

val start : 'a t -> 'a t
(** [Connection.start c] starts the connection [c] running on a separate thread.
 *  Raises {!ClientServer.Invalid_state} if the connection is already started. *)

val send : 'a t -> 'a -> unit
(** [Connection.send c m] sends the message [m] to the connection [c].  Raises
 *  {!ClientServer.Invalid_state} if the connection is not started. *)

val stop : 'a t -> 'a t
(** [Connection.stop c] stops the running connection [c].  Raises
 *  {!ClientServer.Invalid_state} if the connection is already stopped. *)

