open ClientServer
type 'a t = {
  started : bool;
  eb : 'a message EventBroker.t;
  id : int;
  acceptor : 'a Acceptor.t;
  connections : 'a Connection.t list;
  connect : 'a Connection.t -> 'a Connection.t;
  deserialize : in_channel -> 'a;
  handle : 'a -> 'a option;
  serialize : out_channel -> 'a -> unit;
  disconnect : 'a Connection.t -> 'a Connection.t
}
let rec thread_func s = match EventBroker.poll s.eb s.id with
| Some m -> (
  match m with
  | Quit -> ()
  | Connect fd ->
    let c = Connection.create s.eb s.id fd in
    let c = Connection.serializer (Some s.serialize) c in
    let c = Connection.handler (Some s.handle) c in
    let c = Connection.deserializer (Some s.deserialize) c in
    let c = Connection.start c in
    let c = s.connect c in
    thread_func { s with connections = (c::s.connections) }
  | Disconnect id ->
    let c =
      List.find (fun c ->
        if (Connection.id c) = id
        then true
        else false)
      s.connections
    in
    let c = s.disconnect c in
    thread_func { s with connections = (List.filter (fun c ->
      if (Connection.id c) = id
      then false
      else true
    ) s.connections) }
  | Join t ->
    Thread.join t;
    thread_func s
  | _ -> thread_func s
)
| None -> thread_func s
let create eb fd =
  let id = EventBroker.channel eb in
  {
    started = false;
    eb = eb;
    id = id;
    acceptor = Acceptor.create eb id fd;
    connections = [];
    connect = (fun c -> c);
    deserialize = (fun ic -> (Marshal.from_channel ic : 'a));
    handle = (fun m -> None);
    serialize = (fun oc -> fun m -> Marshal.to_channel oc m []; flush oc);
    disconnect = (fun c -> c);
  }
let connect f s = match s.started, f with
| false, Some f -> { s with connect = f }
| false, None -> { s with connect = (fun c -> c) }
| true, _ -> raise (Invalid_state "Server already active")
let serializer f s = match s.started, f with
| false, Some f -> { s with serialize = f }
| false, None -> { s with serialize = (fun oc -> fun m ->
    Marshal.to_channel oc m [];
    flush oc
)}
| true, _ -> raise (Invalid_state "Server already active")
let handler f s = match s.started, f with
| false, Some f -> { s with handle = f }
| false, None -> { s with handle = (fun m -> None) }
| true, _ -> raise (Invalid_state "Server already active")
let deserializer f s = match s.started, f with
| false, Some f -> { s with deserialize = f }
| false, None -> { s with deserialize = (fun ic ->
    (Marshal.from_channel ic : 'a)
)}
| true, _ -> raise (Invalid_state "Server already active")
let disconnect f s = match s.started, f with
| false, Some f -> { s with disconnect = f }
| false, None -> { s with disconnect = (fun c -> c) }
| true, _ -> raise (Invalid_state "Server already active")
let start s = match s.started with
| true -> raise (Invalid_state "Server already active")
| false ->
  let _ = Thread.create thread_func s in
  let a = Acceptor.start s.acceptor in
  { s with acceptor = a; started = true }
let connections s = s.connections
let broadcast s m = match s.started with
| true -> List.iter (fun c -> Connection.send c m) (connections s)
| false -> raise (Invalid_state "Server not active")
let stop s = match s.started with
| false -> raise (Invalid_state "Server not active")
| true ->
  EventBroker.send s.eb s.id Quit;
  { s with started = false }

