(**
 * Functions for controlling the server.
 * @author Jonathan Bryant
 *)

type 'a t
(** The abstract type of servers *)

val create : 'a ClientServer.message EventBroker.t -> Unix.file_descr -> 'a t
(** [Server.create eb fd] creates a new server using the {!EventBroker} [eb] and
 *  the UNIX file descriptor [fd]. *)

val connect : ('a Connection.t -> 'a Connection.t) option -> 'a t -> 'a t
(** [Server.connect f s] sets the function to be applied by the server [s] to
 *  all new connections to the function [f].  This function is applied
 *  immediately after the connection is created and initialized, but before the
 *  connection is started.  If this function is applied to a [None] value, then
 *  the server reverts to its default behavior.  Raises
 *  {!ClientServer.Invalid_state} if this function is applied to a running
 *  server. *)

val deserializer : (in_channel -> 'a) option -> 'a t -> 'a t
(** [Server.deserializer f s] sets the deserialization function of the server
 *  [s] to the function [f].  This deserialization function is inheritied by all
 *  of the connections the server creates (but can be changed in the connect
 *  lifecycle function).  If this function is applied to a [None] value, then
 *  the server reverts to its default behavior.  Raises
 *  {!ClientServer.Invalid_state} if this function is applied to a running
 *  server. *)

val handler : ('a -> 'a option) option -> 'a t -> 'a t
(** [Server.handler f s] sets the handling function of the server [s] to the
 *  function [f].  This function inherited by all of the connections that server
 *  creates (but can be changed in the connect lifecycle function).  If [f]
 *  returns a [Some m] value, then [m] is sent back to the client.  If this
 *  function is applied to a [None] value, then the server reverts to its
 *  default behavior.  Raises {!ClientServer.Invalid_state} if this function is
 *  applied to a running server. *)

val serializer : (out_channel -> 'a -> unit) option -> 'a t -> 'a t
(** [Server.serializer f s] sets the serialization function of the server [s] to
 *  the function [f].  This function is inherited by all of the connections this
 *  server makes (but can be changed in the connect lifecycle function).  If
 *  this function is applied to a [None] value, then the server reverts to its
 *  default behavior.  Raises {!ClientServer.Invalid_state} if this function is
 *  applied to a running server. *)

val disconnect : ('a Connection.t -> 'a Connection.t) option -> 'a t -> 'a t
(** [Server.disconnect f s] sets the function to be applied by the server [s] to
 *  all connections when they die to [f].  The function [f] is applied to the
 *  connection immediately after the connection has been told to quit or the
 *  connection dies ([End_of_file] exceptions raised from this function are not
 *  caught by this library), but before the connection has been stopped.  If
 *  this function is applied to a [None] value, then then server reverts to its
 *  default behavior.  Raises {!ClientServer.Invalid_state} if this function is
 *  applied to a running server. *)

val start : 'a t -> 'a t
(** [Server.start s] starts the server [s] on a separate thread.  Raises
 *  {!ClientServer.Invalid_state} if this function is applied to a running
 *  server. *)

val connections : 'a t -> 'a Connection.t list
(** [Server.connections s] gets a list of the currently connected clients.
 *  Currently, this function is inoperative and simply returns an empty list.
 *  Raises {!ClientServer.Invalid_state} if this function is applied to a
 *  stopped server. *)

val broadcast : 'a t -> 'a -> unit
(** [Server.broadcast s m] sends the message [m] to all of the clients connected
 *  to the server [s].  Currently, this function is inoperative as it relies on
 *  the function {!Server.connections}.  Raises {!ClientServer.Invalid_state} if
 *  this function is applied to a stopped server. *)

val stop : 'a t -> 'a t
(** [Server.stop s] stops the running server [s] and severs all connections.
 *  Raises {!ClientServer.Invalid_state} if this function is applied to a
 *  stopped server. *)


